==> llm-docs-0.1.2.tgz: package/docs/security/validation.md <==
---
id: security-validation
---
# Data Validation and Sanitization Guide

<!-- title: Data Validation and Sanitization Guide -->
<!-- description: Practical guide for data validation and sanitization in the Bluefly Platform -->
<!-- lastUpdated: 2024-03-19 -->

## Overview

This guide covers data validation and sanitization for the Bluefly Platform, including input validation, output encoding, data sanitization, and security measures.

## Input Validation

### Validation Service

```typescript
// src/security/validation/service.ts
import { ValidationRule, ValidationResult, ValidationError } from './types';
import { ValidationMetrics } from './metrics';
import { ValidationLogger } from './logger';

export class ValidationService {
  constructor(
    private metrics: ValidationMetrics,
    private logger: ValidationLogger,
    private options: ValidationOptions
  ) {}

  async validate(
    data: any,
    rules: ValidationRule[]
  ): Promise<ValidationResult> {
    const startTime = Date.now();

    try {
      // Validate data
      const errors = await this.validateRules(data, rules);

      // Record metrics
      this.recordValidation(data, rules, errors, startTime);

      // Log validation
      await this.logValidation(data, rules, errors);

      return {
        valid: errors.length === 0,
        errors
      };
    } catch (error) {
      // Record error
      this.recordValidationError(
        data,
        rules,
        error,
        startTime
      );

      // Log error
      await this.logValidationError(data, rules, error);

      throw error;
    }
  }

  private async validateRules(
    data: any,
    rules: ValidationRule[]
  ): Promise<ValidationError[]> {
    const errors: ValidationError[] = [];

    // Validate all rules
    for (const rule of rules) {
      const ruleErrors = await this.validateRule(
        data,
        rule
      );
      errors.push(...ruleErrors);
    }

    return errors;
  }

  private async validateRule(
    data: any,
    rule: ValidationRule
  ): Promise<ValidationError[]> {
    const errors: ValidationError[] = [];

    // Get value
    const value = this.getValue(data, rule.path);

    // Check required
    if (rule.required && value === undefined) {
      errors.push({
        path: rule.path,
        code: 'REQUIRED',
        message: 'Field is required'
      });
      return errors;
    }

    // Skip if optional and undefined
    if (!rule.required && value === undefined) {
      return errors;
    }

    // Validate type
    if (rule.type && !this.validateType(value, rule.type)) {
      errors.push({
        path: rule.path,
        code: 'INVALID_TYPE',
        message: `Expected type: ${rule.type}`
      });
      return errors;
    }

    // Validate format
    if (rule.format && !this.validateFormat(value, rule.format)) {
      errors.push({
        path: rule.path,
        code: 'INVALID_FORMAT',
        message: `Invalid format: ${rule.format}`
      });
      return errors;
    }

    // Validate pattern
    if (rule.pattern && !this.validatePattern(value, rule.pattern)) {
      errors.push({
        path: rule.path,
        code: 'INVALID_PATTERN',
        message: `Invalid pattern: ${rule.pattern}`
      });
      return errors;
    }

    // Validate range
    if (rule.range && !this.validateRange(value, rule.range)) {
      errors.push({
        path: rule.path,
        code: 'INVALID_RANGE',
        message: `Invalid range: ${rule.range}`
      });
      return errors;
    }

    // Validate length
    if (rule.length && !this.validateLength(value, rule.length)) {
      errors.push({
        path: rule.path,
        code: 'INVALID_LENGTH',
        message: `Invalid length: ${rule.length}`
      });
      return errors;
    }

    // Validate enum
    if (rule.enum && !this.validateEnum(value, rule.enum)) {
      errors.push({
        path: rule.path,
        code: 'INVALID_ENUM',
        message: `Invalid enum: ${rule.enum}`
      });
      return errors;
    }

    // Validate custom
    if (rule.validate) {
      const customErrors = await rule.validate(value);
      errors.push(...customErrors);
    }

    return errors;
  }

  private getValue(data: any, path: string): any {
    return path.split('.').reduce(
      (obj, key) => (obj ? obj[key] : undefined),
      data
    );
  }

  private validateType(value: any, type: string): boolean {
    switch (type) {
      case 'string':
        return typeof value === 'string';

      case 'number':
        return typeof value === 'number' && !isNaN(value);

      case 'boolean':
        return typeof value === 'boolean';

      case 'array':
        return Array.isArray(value);

      case 'object':
        return typeof value === 'object' && value !== null;

      case 'date':
        return value instanceof Date && !isNaN(value.getTime());

      default:
        return false;
    }
  }

  private validateFormat(
    value: string,
    format: string
  ): boolean {
    switch (format) {
      case 'email':
        return /^[^\s@]+@[^\s@]+\.[^\s@]+$/.test(value);

      case 'url':
        try {
          new URL(value);
          return true;
        } catch {
          return false;
        }

      case 'uuid':
        return /^[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}$/i.test(
          value
        );

      case 'date':
        return !isNaN(Date.parse(value));

      case 'ipv4':
        return /^(\d{1,3}\.){3}\d{1,3}$/.test(value);

      case 'ipv6':
        return /^([0-9a-f]{1,4}:){7}[0-9a-f]{1,4}$/i.test(
          value
        );

      default:
        return false;
    }
  }

  private validatePattern(
    value: string,
    pattern: string
  ): boolean {
    try {
      return new RegExp(pattern).test(value);
    } catch {
      return false;
    }
  }

  private validateRange(
    value: number,
    range: {
      min?: number;
      max?: number;
    }
  ): boolean {
    if (range.min !== undefined && value < range.min) {
      return false;
    }

    if (range.max !== undefined && value > range.max) {
      return false;
    }

    return true;
  }

  private validateLength(
    value: string | any[],
    length: {
      min?: number;
      max?: number;
    }
  ): boolean {
    const len = value.length;

    if (length.min !== undefined && len < length.min) {
      return false;
    }

    if (length.max !== undefined && len > length.max) {
      return false;
    }

    return true;
  }

  private validateEnum(
    value: any,
    enum_: any[]
  ): boolean {
    return enum_.includes(value);
  }

  private recordValidation(
    data: any,
    rules: ValidationRule[],
    errors: ValidationError[],
    startTime: number
  ): void {
    const duration = Date.now() - startTime;

    this.metrics.recordValidation({
      rules: rules.length,
      errors: errors.length,
      duration
    });
  }

  private recordValidationError(
    data: any,
    rules: ValidationRule[],
    error: Error,
    startTime: number
  ): void {
    const duration = Date.now() - startTime;

    this.metrics.recordValidationError({
      rules: rules.length,
      error: error.name,
      duration
    });
  }

  private async logValidation(
    data: any,
    rules: ValidationRule[],
    errors: ValidationError[]
  ): Promise<void> {
    await this.logger.logValidation({
      data,
      rules,
      errors,
      timestamp: new Date()
    });
  }

  private async logValidationError(
    data: any,
    rules: ValidationRule[],
    error: Error
  ): Promise<void> {
    await this.logger.logValidationError({
      data,
      rules,
      error,
      timestamp: new Date()
    });
  }
}

interface ValidationOptions {
  environment: string;
  service: string;
  version: string;
}

interface ValidationRule {
  path: string;
  required?: boolean;
  type?: string;
  format?: string;
  pattern?: string;
  range?: {
    min?: number;
    max?: number;
  };
  length?: {
    min?: number;
    max?: number;
  };
  enum?: any[];
  validate?: (value: any) => Promise<ValidationError[]>;
}

interface ValidationResult {
  valid: boolean;
  errors: ValidationError[];
}

interface ValidationError {
  path: string;
  code: string;
  message: string;
}
```

### Sanitization Service

```typescript
// src/security/sanitization/service.ts
import { SanitizationRule, SanitizationResult } from './types';
import { ValidationMetrics } from '../validation/metrics';
import { ValidationLogger } from '../validation/logger';

export class SanitizationService {
  constructor(
    private metrics: ValidationMetrics,
    private logger: ValidationLogger,
    private options: SanitizationOptions
  ) {}

  async sanitize(
    data: any,
    rules: SanitizationRule[]
  ): Promise<SanitizationResult> {
    const startTime = Date.now();

    try {
      // Sanitize data
      const sanitized = await this.sanitizeRules(data, rules);

      // Record metrics
      this.recordSanitization(data, rules, sanitized, startTime);

      // Log sanitization
      await this.logSanitization(data, rules, sanitized);

      return {
        data: sanitized,
        changes: this.getChanges(data, sanitized)
      };
    } catch (error) {
      // Record error
      this.recordSanitizationError(
        data,
        rules,
        error,
        startTime
      );

      // Log error
      await this.logSanitizationError(data, rules, error);

      throw error;
    }
  }

  private async sanitizeRules(
    data: any,
    rules: SanitizationRule[]
  ): Promise<any> {
    let sanitized = { ...data };

    // Apply all rules
    for (const rule of rules) {
      sanitized = await this.sanitizeRule(
        sanitized,
        rule
      );
    }

    return sanitized;
  }

  private async sanitizeRule(
    data: any,
    rule: SanitizationRule
  ): Promise<any> {
    const sanitized = { ...data };

    // Get value
    const value = this.getValue(sanitized, rule.path);

    // Skip if undefined
    if (value === undefined) {
      return sanitized;
    }

    // Sanitize value
    const sanitizedValue = await this.sanitizeValue(
      value,
      rule
    );

    // Set value
    this.setValue(sanitized, rule.path, sanitizedValue);

    return sanitized;
  }

  private async sanitizeValue(
    value: any,
    rule: SanitizationRule
  ): Promise<any> {
    // Apply sanitizers
    let sanitized = value;

    if (rule.trim) {
      sanitized = this.trim(sanitized);
    }

    if (rule.lowercase) {
      sanitized = this.lowercase(sanitized);
    }

    if (rule.uppercase) {
      sanitized = this.uppercase(sanitized);
    }

    if (rule.escape) {
      sanitized = this.escape(sanitized);
    }

    if (rule.strip) {
      sanitized = this.strip(sanitized);
    }

    if (rule.replace) {
      sanitized = this.replace(sanitized, rule.replace);
    }

    if (rule.transform) {
      sanitized = await rule.transform(sanitized);
    }

    return sanitized;
  }

  private getValue(data: any, path: string): any {
    return path.split('.').reduce(
      (obj, key) => (obj ? obj[key] : undefined),
      data
    );
  }

  private setValue(
    data: any,
    path: string,
    value: any
  ): void {
    const keys = path.split('.');
    const lastKey = keys.pop()!;

    const obj = keys.reduce(
      (obj, key) => (obj[key] = obj[key] || {}),
      data
    );

    obj[lastKey] = value;
  }

  private trim(value: string): string {
    return typeof value === 'string'
      ? value.trim()
      : value;
  }

  private lowercase(value: string): string {
    return typeof value === 'string'
      ? value.toLowerCase()
      : value;
  }

  private uppercase(value: string): string {
    return typeof value === 'string'
      ? value.toUpperCase()
      : value;
  }

  private escape(value: string): string {
    if (typeof value !== 'string') {
      return value;
    }

    return value
      .replace(/&/g, '&amp;')
      .replace(/</g, '&lt;')
      .replace(/>/g, '&gt;')
      .replace(/"/g, '&quot;')
      .replace(/'/g, '&#039;');
  }

  private strip(value: string): string {
    if (typeof value !== 'string') {
      return value;
    }

    return value.replace(/[^\w\s]/gi, '');
  }

  private replace(
    value: string,
    replace: {
      pattern: string;
      replacement: string;
    }
  ): string {
    if (typeof value !== 'string') {
      return value;
    }

    try {
      return value.replace(
        new RegExp(replace.pattern, 'g'),
        replace.replacement
      );
    } catch {
      return value;
    }
  }

  private getChanges(
    original: any,
    sanitized: any
  ): string[] {
    const changes: string[] = [];

    this.compareObjects(original, sanitized, '', changes);

    return changes;
  }

  private compareObjects(
    original: any,
    sanitized: any,
    path: string,
    changes: string[]
  ): void {
    if (
      typeof original !== 'object' ||
      typeof sanitized !== 'object' ||
      original === null ||
      sanitized === null
    ) {
      if (original !== sanitized) {
        changes.push(path);
      }
      return;
    }

    const keys = new Set([
      ...Object.keys(original),
      ...Object.keys(sanitized)
    ]);

    for (const key of keys) {
      const newPath = path ? `${path}.${key}` : key;

      if (
        !(key in original) ||
        !(key in sanitized) ||
        original[key] !== sanitized[key]
      ) {
        this.compareObjects(
          original[key],
          sanitized[key],
          newPath,
          changes
        );
      }
    }
  }

  private recordSanitization(
    data: any,
    rules: SanitizationRule[],
    sanitized: any,
    startTime: number
  ): void {
    const duration = Date.now() - startTime;

    this.metrics.recordSanitization({
      rules: rules.length,
      changes: this.getChanges(data, sanitized).length,
      duration
    });
  }

  private recordSanitizationError(
    data: any,
    rules: SanitizationRule[],
    error: Error,
    startTime: number
  ): void {
    const duration = Date.now() - startTime;

    this.metrics.recordSanitizationError({
      rules: rules.length,
      error: error.name,
      duration
    });
  }

  private async logSanitization(
    data: any,
    rules: SanitizationRule[],
    sanitized: any
  ): Promise<void> {
    await this.logger.logSanitization({
      data,
      rules,
      sanitized,
      changes: this.getChanges(data, sanitized),
      timestamp: new Date()
    });
  }

  private async logSanitizationError(
    data: any,
    rules: SanitizationRule[],
    error: Error
  ): Promise<void> {
    await this.logger.logSanitizationError({
      data,
      rules,
      error,
      timestamp: new Date()
    });
  }
}

interface SanitizationOptions {
  environment: string;
  service: string;
  version: string;
}

interface SanitizationRule {
  path: string;
  trim?: boolean;
  lowercase?: boolean;
  uppercase?: boolean;
  escape?: boolean;
  strip?: boolean;
  replace?: {
    pattern: string;
    replacement: string;
  };
  transform?: (value: any) => Promise<any>;
}

interface SanitizationResult {
  data: any;
  changes: string[];
}
```

## Best Practices

1. **Input Validation**
   - Define rules
   - Validate types
   - Check formats
   - Set limits
   - Handle errors

2. **Data Sanitization**
   - Sanitize input
   - Escape output
   - Remove unsafe
   - Transform data
   - Track changes

3. **Security Measures**
   - Validate all input
   - Sanitize all output
   - Use whitelists
   - Set limits
   - Log changes

4. **Error Handling**
   - Validate early
   - Sanitize always
   - Log errors
   - Monitor changes
   - Review logs

## Related Documentation

- [Security Guide](../security/README)
- [API Guide](../api/README)
- [Monitoring Guide](../monitoring/README)
- [Testing Guide](../testing/README)
- [Deployment Guide](../deployment/README)

==> llm-docs-0.1.5.tgz: package/docs/security/validation.md <==
---
id: security-validation
---
# Data Validation and Sanitization Guide

<!-- title: Data Validation and Sanitization Guide -->
<!-- description: Practical guide for data validation and sanitization in the LLM Platform Platform -->
<!-- lastUpdated: 2024-03-19 -->

## Overview

This guide covers data validation and sanitization for the LLM Platform Platform, including input validation, output encoding, data sanitization, and security measures.

## Input Validation

### Validation Service

```typescript
// src/security/validation/service.ts
import { ValidationRule, ValidationResult, ValidationError } from './types';
import { ValidationMetrics } from './metrics';
import { ValidationLogger } from './logger';

export class ValidationService {
  constructor(
    private metrics: ValidationMetrics,
    private logger: ValidationLogger,
    private options: ValidationOptions
  ) {}

  async validate(
    data: any,
    rules: ValidationRule[]
  ): Promise<ValidationResult> {
    const startTime = Date.now();

    try {
      // Validate data
      const errors = await this.validateRules(data, rules);

      // Record metrics
      this.recordValidation(data, rules, errors, startTime);

      // Log validation
      await this.logValidation(data, rules, errors);

      return {
        valid: errors.length === 0,
        errors
      };
    } catch (error) {
      // Record error
      this.recordValidationError(
        data,
        rules,
        error,
        startTime
      );

      // Log error
      await this.logValidationError(data, rules, error);

      throw error;
    }
  }

  private async validateRules(
    data: any,
    rules: ValidationRule[]
  ): Promise<ValidationError[]> {
    const errors: ValidationError[] = [];

    // Validate all rules
    for (const rule of rules) {
      const ruleErrors = await this.validateRule(
        data,
        rule
      );
      errors.push(...ruleErrors);
    }

    return errors;
  }

  private async validateRule(
    data: any,
    rule: ValidationRule
  ): Promise<ValidationError[]> {
    const errors: ValidationError[] = [];

    // Get value
    const value = this.getValue(data, rule.path);

    // Check required
    if (rule.required && value === undefined) {
      errors.push({
        path: rule.path,
        code: 'REQUIRED',
        message: 'Field is required'
      });
      return errors;
    }

    // Skip if optional and undefined
    if (!rule.required && value === undefined) {
      return errors;
    }

    // Validate type
    if (rule.type && !this.validateType(value, rule.type)) {
      errors.push({
        path: rule.path,
        code: 'INVALID_TYPE',
        message: `Expected type: ${rule.type}`
      });
      return errors;
    }

    // Validate format
    if (rule.format && !this.validateFormat(value, rule.format)) {
      errors.push({
        path: rule.path,
        code: 'INVALID_FORMAT',
        message: `Invalid format: ${rule.format}`
      });
      return errors;
    }

    // Validate pattern
    if (rule.pattern && !this.validatePattern(value, rule.pattern)) {
      errors.push({
        path: rule.path,
        code: 'INVALID_PATTERN',
        message: `Invalid pattern: ${rule.pattern}`
      });
      return errors;
    }

    // Validate range
    if (rule.range && !this.validateRange(value, rule.range)) {
      errors.push({
        path: rule.path,
        code: 'INVALID_RANGE',
        message: `Invalid range: ${rule.range}`
      });
      return errors;
    }

    // Validate length
    if (rule.length && !this.validateLength(value, rule.length)) {
      errors.push({
        path: rule.path,
        code: 'INVALID_LENGTH',
        message: `Invalid length: ${rule.length}`
      });
      return errors;
    }

    // Validate enum
    if (rule.enum && !this.validateEnum(value, rule.enum)) {
      errors.push({
        path: rule.path,
        code: 'INVALID_ENUM',
        message: `Invalid enum: ${rule.enum}`
      });
      return errors;
    }

    // Validate custom
    if (rule.validate) {
      const customErrors = await rule.validate(value);
      errors.push(...customErrors);
    }

    return errors;
  }

  private getValue(data: any, path: string): any {
    return path.split('.').reduce(
      (obj, key) => (obj ? obj[key] : undefined),
      data
    );
  }

  private validateType(value: any, type: string): boolean {
    switch (type) {
      case 'string':
        return typeof value === 'string';

      case 'number':
        return typeof value === 'number' && !isNaN(value);

      case 'boolean':
        return typeof value === 'boolean';

      case 'array':
        return Array.isArray(value);

      case 'object':
        return typeof value === 'object' && value !== null;

      case 'date':
        return value instanceof Date && !isNaN(value.getTime());

      default:
        return false;
    }
  }

  private validateFormat(
    value: string,
    format: string
  ): boolean {
    switch (format) {
      case 'email':
        return /^[^\s@]+@[^\s@]+\.[^\s@]+$/.test(value);

      case 'url':
        try {
          new URL(value);
          return true;
        } catch {
          return false;
        }

      case 'uuid':
        return /^[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}$/i.test(
          value
        );

      case 'date':
        return !isNaN(Date.parse(value));

      case 'ipv4':
        return /^(\d{1,3}\.){3}\d{1,3}$/.test(value);

      case 'ipv6':
        return /^([0-9a-f]{1,4}:){7}[0-9a-f]{1,4}$/i.test(
          value
        );

      default:
        return false;
    }
  }

  private validatePattern(
    value: string,
    pattern: string
  ): boolean {
    try {
      return new RegExp(pattern).test(value);
    } catch {
      return false;
    }
  }

  private validateRange(
    value: number,
    range: {
      min?: number;
      max?: number;
    }
  ): boolean {
    if (range.min !== undefined && value < range.min) {
      return false;
    }

    if (range.max !== undefined && value > range.max) {
      return false;
    }

    return true;
  }

  private validateLength(
    value: string | any[],
    length: {
      min?: number;
      max?: number;
    }
  ): boolean {
    const len = value.length;

    if (length.min !== undefined && len < length.min) {
      return false;
    }

    if (length.max !== undefined && len > length.max) {
      return false;
    }

    return true;
  }

  private validateEnum(
    value: any,
    enum_: any[]
  ): boolean {
    return enum_.includes(value);
  }

  private recordValidation(
    data: any,
    rules: ValidationRule[],
    errors: ValidationError[],
    startTime: number
  ): void {
    const duration = Date.now() - startTime;

    this.metrics.recordValidation({
      rules: rules.length,
      errors: errors.length,
      duration
    });
  }

  private recordValidationError(
    data: any,
    rules: ValidationRule[],
    error: Error,
    startTime: number
  ): void {
    const duration = Date.now() - startTime;

    this.metrics.recordValidationError({
      rules: rules.length,
      error: error.name,
      duration
    });
  }

  private async logValidation(
    data: any,
    rules: ValidationRule[],
    errors: ValidationError[]
  ): Promise<void> {
    await this.logger.logValidation({
      data,
      rules,
      errors,
      timestamp: new Date()
    });
  }

  private async logValidationError(
    data: any,
    rules: ValidationRule[],
    error: Error
  ): Promise<void> {
    await this.logger.logValidationError({
      data,
      rules,
      error,
      timestamp: new Date()
    });
  }
}

interface ValidationOptions {
  environment: string;
  service: string;
  version: string;
}

interface ValidationRule {
  path: string;
  required?: boolean;
  type?: string;
  format?: string;
  pattern?: string;
  range?: {
    min?: number;
    max?: number;
  };
  length?: {
    min?: number;
    max?: number;
  };
  enum?: any[];
  validate?: (value: any) => Promise<ValidationError[]>;
}

interface ValidationResult {
  valid: boolean;
  errors: ValidationError[];
}

interface ValidationError {
  path: string;
  code: string;
  message: string;
}
```

### Sanitization Service

```typescript
// src/security/sanitization/service.ts
import { SanitizationRule, SanitizationResult } from './types';
import { ValidationMetrics } from '../validation/metrics';
import { ValidationLogger } from '../validation/logger';

export class SanitizationService {
  constructor(
    private metrics: ValidationMetrics,
    private logger: ValidationLogger,
    private options: SanitizationOptions
  ) {}

  async sanitize(
    data: any,
    rules: SanitizationRule[]
  ): Promise<SanitizationResult> {
    const startTime = Date.now();

    try {
      // Sanitize data
      const sanitized = await this.sanitizeRules(data, rules);

      // Record metrics
      this.recordSanitization(data, rules, sanitized, startTime);

      // Log sanitization
      await this.logSanitization(data, rules, sanitized);

      return {
        data: sanitized,
        changes: this.getChanges(data, sanitized)
      };
    } catch (error) {
      // Record error
      this.recordSanitizationError(
        data,
        rules,
        error,
        startTime
      );

      // Log error
      await this.logSanitizationError(data, rules, error);

      throw error;
    }
  }

  private async sanitizeRules(
    data: any,
    rules: SanitizationRule[]
  ): Promise<any> {
    let sanitized = { ...data };

    // Apply all rules
    for (const rule of rules) {
      sanitized = await this.sanitizeRule(
        sanitized,
        rule
      );
    }

    return sanitized;
  }

  private async sanitizeRule(
    data: any,
    rule: SanitizationRule
  ): Promise<any> {
    const sanitized = { ...data };

    // Get value
    const value = this.getValue(sanitized, rule.path);

    // Skip if undefined
    if (value === undefined) {
      return sanitized;
    }

    // Sanitize value
    const sanitizedValue = await this.sanitizeValue(
      value,
      rule
    );

    // Set value
    this.setValue(sanitized, rule.path, sanitizedValue);

    return sanitized;
  }

  private async sanitizeValue(
    value: any,
    rule: SanitizationRule
  ): Promise<any> {
    // Apply sanitizers
    let sanitized = value;

    if (rule.trim) {
      sanitized = this.trim(sanitized);
    }

    if (rule.lowercase) {
      sanitized = this.lowercase(sanitized);
    }

    if (rule.uppercase) {
      sanitized = this.uppercase(sanitized);
    }

    if (rule.escape) {
      sanitized = this.escape(sanitized);
    }

    if (rule.strip) {
      sanitized = this.strip(sanitized);
    }

    if (rule.replace) {
      sanitized = this.replace(sanitized, rule.replace);
    }

    if (rule.transform) {
      sanitized = await rule.transform(sanitized);
    }

    return sanitized;
  }

  private getValue(data: any, path: string): any {
    return path.split('.').reduce(
      (obj, key) => (obj ? obj[key] : undefined),
      data
    );
  }

  private setValue(
    data: any,
    path: string,
    value: any
  ): void {
    const keys = path.split('.');
    const lastKey = keys.pop()!;

    const obj = keys.reduce(
      (obj, key) => (obj[key] = obj[key] || {}),
      data
    );

    obj[lastKey] = value;
  }

  private trim(value: string): string {
    return typeof value === 'string'
      ? value.trim()
      : value;
  }

  private lowercase(value: string): string {
    return typeof value === 'string'
      ? value.toLowerCase()
      : value;
  }

  private uppercase(value: string): string {
    return typeof value === 'string'
      ? value.toUpperCase()
      : value;
  }

  private escape(value: string): string {
    if (typeof value !== 'string') {
      return value;
    }

    return value
      .replace(/&/g, '&amp;')
      .replace(/</g, '&lt;')
      .replace(/>/g, '&gt;')
      .replace(/"/g, '&quot;')
      .replace(/'/g, '&#039;');
  }

  private strip(value: string): string {
    if (typeof value !== 'string') {
      return value;
    }

    return value.replace(/[^\w\s]/gi, '');
  }

  private replace(
    value: string,
    replace: {
      pattern: string;
      replacement: string;
    }
  ): string {
    if (typeof value !== 'string') {
      return value;
    }

    try {
      return value.replace(
        new RegExp(replace.pattern, 'g'),
        replace.replacement
      );
    } catch {
      return value;
    }
  }

  private getChanges(
    original: any,
    sanitized: any
  ): string[] {
    const changes: string[] = [];

    this.compareObjects(original, sanitized, '', changes);

    return changes;
  }

  private compareObjects(
    original: any,
    sanitized: any,
    path: string,
    changes: string[]
  ): void {
    if (
      typeof original !== 'object' ||
      typeof sanitized !== 'object' ||
      original === null ||
      sanitized === null
    ) {
      if (original !== sanitized) {
        changes.push(path);
      }
      return;
    }

    const keys = new Set([
      ...Object.keys(original),
      ...Object.keys(sanitized)
    ]);

    for (const key of keys) {
      const newPath = path ? `${path}.${key}` : key;

      if (
        !(key in original) ||
        !(key in sanitized) ||
        original[key] !== sanitized[key]
      ) {
        this.compareObjects(
          original[key],
          sanitized[key],
          newPath,
          changes
        );
      }
    }
  }

  private recordSanitization(
    data: any,
    rules: SanitizationRule[],
    sanitized: any,
    startTime: number
  ): void {
    const duration = Date.now() - startTime;

    this.metrics.recordSanitization({
      rules: rules.length,
      changes: this.getChanges(data, sanitized).length,
      duration
    });
  }

  private recordSanitizationError(
    data: any,
    rules: SanitizationRule[],
    error: Error,
    startTime: number
  ): void {
    const duration = Date.now() - startTime;

    this.metrics.recordSanitizationError({
      rules: rules.length,
      error: error.name,
      duration
    });
  }

  private async logSanitization(
    data: any,
    rules: SanitizationRule[],
    sanitized: any
  ): Promise<void> {
    await this.logger.logSanitization({
      data,
      rules,
      sanitized,
      changes: this.getChanges(data, sanitized),
      timestamp: new Date()
    });
  }

  private async logSanitizationError(
    data: any,
    rules: SanitizationRule[],
    error: Error
  ): Promise<void> {
    await this.logger.logSanitizationError({
      data,
      rules,
      error,
      timestamp: new Date()
    });
  }
}

interface SanitizationOptions {
  environment: string;
  service: string;
  version: string;
}

interface SanitizationRule {
  path: string;
  trim?: boolean;
  lowercase?: boolean;
  uppercase?: boolean;
  escape?: boolean;
  strip?: boolean;
  replace?: {
    pattern: string;
    replacement: string;
  };
  transform?: (value: any) => Promise<any>;
}

interface SanitizationResult {
  data: any;
  changes: string[];
}
```

## Best Practices

1. **Input Validation**
   - Define rules
   - Validate types
   - Check formats
   - Set limits
   - Handle errors

2. **Data Sanitization**
   - Sanitize input
   - Escape output
   - Remove unsafe
   - Transform data
   - Track changes

3. **Security Measures**
   - Validate all input
   - Sanitize all output
   - Use whitelists
   - Set limits
   - Log changes

4. **Error Handling**
   - Validate early
   - Sanitize always
   - Log errors
   - Monitor changes
   - Review logs

## Related Documentation

- [Security Guide](../security/README)
- [API Guide](../api/README)
- [Monitoring Guide](../monitoring/README)
- [Testing Guide](../testing/README)
- [Deployment Guide](../deployment/README)
